# HEAT_AND_MOISTURE_TRANSFER_2.tcl  -*- TCL -*-
#
# tcl file for SIFEL input file printing (TRFEL - HEAT AND MOISTURE TRANSFER)
# Kunzel's approach - 2 unknowns RH relative humidity and T temperature
#
# Tomas Krejci, 10.10.2005
#---------------------------------------------------------------------------
#
# This file is written in TCL language 
# For more information about TCL look at: http://www.sunlabs.com/research/tcl/
#                                    at: http://www.tcl.tk
#
# At least two procs must be in this file:
#
#    InitGIDProject dir - Will be called whenever a project is begun to be used.
#           where dir is the project's directory
#    EndGIDProject - Will be called whenever a project ends to be used.
#
# For more information about GID internals, check the program scripts.
#---------------------------------------------------------------------------

proc InitGIDProject { dir } {

    CreateMenu "SIFEL-preprocessor" "PRE"
    InsertMenuOption "SIFEL-preprocessor" "Help" 0 "WriteCalcFileHelp" "PRE" 
    InsertMenuOption "SIFEL-preprocessor" "SIFEL Units" 1 "WriteCalcFileUnits" "PRE" 
    InsertMenuOption "SIFEL-preprocessor" "----" 2 "" "PRE" 
    InsertMenuOption "SIFEL-preprocessor" "Write SIFEL input file" 3 "WriteCalcFile" "PRE" 
    InsertMenuOption "SIFEL-preprocessor" "----" 4 "" "PRE" 
    InsertMenuOption "SIFEL-preprocessor" "Open new problem type" 5 "OpenProbType" "PRE" 
    UpdateMenus

    CreateWindow
}

proc EndGIDProject {} {
}

proc CreateWindow {} {
    
    set w .gid.heat_transfer

    InitWindow $w "METR COUPLED THERMO-HYDRO-MECHANICAL PROBLEM" METR_COUPLED_THERMO-HYDRO-MECHANICAL_PROBLEM "" \
	    "" 1

    frame $w.top
    label $w.top.title_text -text " GiD module - SIFEL input file "
   
    frame $w.information  -relief ridge -bd 2 
    label $w.information.problem1     -text " Problem type:    General problem - General approach 
                                              \ "
    label $w.information.part         -text " SIFEL module:    METR,TRFEL,MEFEL "
    label $w.information.line         -text " "
    label $w.information.copyright    -text " Copyright (c) by SIFEL group, September 2005, http://cml.fsv.cvut.cz/~sifel/ "

    frame $w.bottom
    button $w.bottom.start -text "CONTINUE" \
           -height 1 -width 14 -command "destroy $w"


    pack $w.top.title_text -pady 10
    pack $w.information.problem1 $w.information.part \
         $w.information.line $w.information.copyright -side top -anchor w
    
    pack $w.bottom.start -side left -anchor center
    pack $w.top 
    pack $w.information -expand yes -fill both
    pack $w.bottom -side top -padx 6 -pady 10 -ipady 2
}

proc WriteCalcFileHelp {} {

    set whelp .gid.help_SIFEL_inputfile

    InitWindow $whelp "HELP - input file" SIFEL_HELP "" \
	    "" 1

    frame $whelp.top
    label $whelp.top.title_text -text " How to create SIFEL input file: "
   
    frame $whelp.information  -relief ridge -bd 2 
    label $whelp.information.problem     -text " 1. Set problem data in Data>Problem Data "
    label $whelp.information.domain      -text " 2. Create domain "
    label $whelp.information.materials   -text " 3. Set materials in Data>Materials "
    label $whelp.information.conditions  -text " 4. Set boundary and initial conditions in Data>Conditions "
    label $whelp.information.mesh        -text " 5. Generate mesh "
    label $whelp.information.write       -text " 6. Write SIFEL input file in File>Export>Calculation file \
                                                 or in SIFEL-preprocessor>Write SIFEL input file"

    frame $whelp.bottom
    button $whelp.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $whelp"


    pack $whelp.top.title_text -pady 10
    pack $whelp.information.problem $whelp.information.domain $whelp.information.materials \
         $whelp.information.conditions $whelp.information.mesh $whelp.information.write -side top -anchor w
    
    pack $whelp.bottom.help -side left -anchor center
    pack $whelp.top 
    pack $whelp.information -expand yes -fill both
    pack $whelp.bottom -side top -padx 6 -pady 10 -ipady 2
}


proc WriteCalcFileUnits {} {

    set wunits .gid.units_SIFEL_inputfile

    InitWindow $wunits "SIFEL - Basic units" SIFEL_UNITS "" \
	    "" 1

    frame $wunits.top
    label $wunits.top.title_text -text " Basic Units in TRFEL problem: "
   
    frame $wunits.information  -relief ridge -bd 2 
    label $wunits.information.coord       -text " Length or Distance (Coordinates) - meter (m) "
    label $wunits.information.temper      -text " Temperature - Kelvins (K) or degrees of Celsius (C) "
    label $wunits.information.density     -text " Density - kilogram per cubic meter (kg/m^3)"
    label $wunits.information.cond        -text " Thermal Conductivity - Joule per kilogram and Kelvin (J/(kg.K))"
    label $wunits.information.sheat       -text " Specific Heat -  Watt per meter and Kelvin (W/(m.K))"
    label $wunits.information.source      -text " Heat Source - Watt per cubic meter (W/m^3) "
    label $wunits.information.line        -text " "
    label $wunits.information.humid       -text " Realtive humity - (-) "
    label $wunits.information.vappress    -text " Vater vapor pressure - Pascal (Pa) "
    label $wunits.information.volumemoist -text " Moisture volume content - kilogram per cubic meter (kg/m^3) "
    label $wunits.information.weightmoist -text " Moisture weight content - kilogram per kilogram (kg/kg) "

    frame $wunits.bottom
    button $wunits.bottom.units -text "OK" \
           -height 1 -width 14 -command "destroy $wunits"


    pack $wunits.top.title_text -pady 10
    pack $wunits.information.coord $wunits.information.line $wunits.information.temper $wunits.information.density \
         $wunits.information.cond $wunits.information.sheat $wunits.information.source $wunits.information.line \
         $wunits.information.humid $wunits.information.vappress $wunits.information.volumemoist $wunits.information.weightmoist \
         -side top -anchor w
    
    pack $wunits.bottom.units -side left -anchor center
    pack $wunits.top 
    pack $wunits.information -expand yes -fill both
    pack $wunits.bottom -side top -padx 6 -pady 10 -ipady 2
}

proc WriteCalcFile {} {
    
    .central.s process escape escape escape escape Files WriteCalcFile
    
}


proc OpenProbType {} {
    
    .central.s process escape escape escape escape data defaults ProblemType
    
}

proc ErrorMat {} {

    set werr1 .gid.help_err_mat

    InitWindow $werr1 "Error - Materials" ERR_MAT "" \
	    "" 1

    frame $werr1.top
    label $werr1.top.title_text -text " No material assigned!!! "
   
    frame $werr1.information  -relief ridge -bd 2 
    label $werr1.information.problem     -text " No material assigned. Try it again! "

    frame $werr1.bottom
    button $werr1.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $werr1"


    pack $werr1.top.title_text -pady 10
    pack $werr1.information.problem -side top -anchor w
    
    pack $werr1.bottom.help -side left -anchor center
    pack $werr1.top 
    pack $werr1.information -expand yes -fill both
    pack $werr1.bottom -side top -padx 6 -pady 10 -ipady 2

    return ""
}


proc ErrorInitRH {} {

    set werr2 .gid.help_err_initrh

    InitWindow $werr2 "Error - Initial condition" ERR_INIT_RH "" \
	    "" 1

    frame $werr2.top
    label $werr2.top.title_text -text " No initial value assigned!!! "
   
    frame $werr2.information  -relief ridge -bd 2 
    label $werr2.information.problem     -text " No initial value for relative humidity assigned or all values set 0.0! "

    frame $werr2.bottom
    button $werr2.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $werr2"


    pack $werr2.top.title_text -pady 10
    pack $werr2.information.problem -side top -anchor w
    
    pack $werr2.bottom.help -side left -anchor center
    pack $werr2.top 
    pack $werr2.information -expand yes -fill both
    pack $werr2.bottom -side top -padx 6 -pady 10 -ipady 2

    return ""
}


proc ErrorInitT {} {

    set werr3 .gid.help_err_initt

    InitWindow $werr3 "Error - Initial condition" ERR_INIT_T "" \
	    "" 1

    frame $werr3.top
    label $werr3.top.title_text -text " No initial value assigned!!! "
   
    frame $werr3.information  -relief ridge -bd 2 
    label $werr3.information.problem     -text " No initial value for temperature assigned or all values set 0.0! "

    frame $werr3.bottom
    button $werr3.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $werr3"


    pack $werr3.top.title_text -pady 10
    pack $werr3.information.problem -side top -anchor w
    
    pack $werr3.bottom.help -side left -anchor center
    pack $werr3.top 
    pack $werr3.information -expand yes -fill both
    pack $werr3.bottom -side top -padx 6 -pady 10 -ipady 2

    return ""
}


proc RenumElementsHex {} {

    set renum .gid.help_renum

    InitWindow $renum "Element Renumbering" RENUM_EL "" \
	    "" 1

    frame $renum.top
    label $renum.top.title_text -text " Renumbering of element conectivity!!! "
   
    frame $renum.information  -relief ridge -bd 2 
    label $renum.information.problem1     -text " Hexahedrals were renumbered according to SIFEL! "
    label $renum.information.problem2     -text " Open new POST-PROCESSOR project for results!"

    frame $renum.bottom
    button $renum.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $renum"


    pack $renum.top.title_text -pady 10
    pack $renum.information.problem1 $renum.information.problem2 -side top -anchor w
    
    pack $renum.bottom.help -side left -anchor center
    pack $renum.top 
    pack $renum.information -expand yes -fill both
    pack $renum.bottom -side top -padx 6 -pady 10 -ipady 2
}


proc RenumElementsTet {} {

    set renumt .gid.help_renumt

    InitWindow $renumt "Element Renumbering" RENUM_EL "" \
	    "" 1

    frame $renumt.top
    label $renumt.top.title_text -text " Renumbering of element conectivity!!! "
   
    frame $renumt.information  -relief ridge -bd 2 
    label $renumt.information.problem1     -text " Tetrahedrals were renumbered according to SIFEL! "
    label $renumt.information.problem2     -text " Open new POST-PROCESSOR project for results!"

    frame $renumt.bottom
    button $renumt.bottom.help -text "OK" \
           -height 1 -width 14 -command "destroy $renumt"


    pack $renumt.top.title_text -pady 10
    pack $renumt.information.problem1 $renumt.information.problem2 -side top -anchor w
    
    pack $renumt.bottom.help -side left -anchor center
    pack $renumt.top 
    pack $renumt.information -expand yes -fill both
    pack $renumt.bottom -side top -padx 6 -pady 10 -ipady 2
}


proc WriteRange {word} {
    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    #writing and reading of range
    set string ""
    set location_hhyphen 0
    set location_comma 0
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i 0} {$i < $Length} {incr i} {
	
	set hyphen [string compare ":" [string index $word $i]]
	
	if {$hyphen == 0} {
	    set location_hhyphen $i
	    set location_comma [expr $location_comma+$hh]
	    #start value reading
	    for {set j $location_comma} {$j < $location_hhyphen} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 0
	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string   $start"
	    }
	} else {
	}
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set location_comma $i
	    set help [expr $location_hhyphen+1]
	    set help_comma [expr $help_comma+1]
	    
	    #end value reading
	    for {set j $help} {$j < $location_comma} {incr j} {
		set print [string index $word $j]
		set end "$end$print"
	    }
	    set hh 1
	    set string "$string $end"
	} else {
	}
	
	
	set start ""
	set end ""
	
    }

    set string "$X $string"
 
    for {set i 0} {$i < $X} {incr i} {
	set string "$string "
    }
    return $string
}

proc WriteList {word} {

    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    
    #writing and reading of list
    set string ""
    set next_location_comma 0
    set location_comma 0
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set next_location_comma $i
	    set location_comma [expr $location_comma+$hh]
	    #start value reading
	    for {set j $location_comma} {$j < $next_location_comma} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 1
	    set location_comma $next_location_comma

	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string   $start"
	    }
	} else {
	}
	
	set start ""
	
    }

    set string "$X $string"
 
    for {set i 0} {$i < $X} {incr i} {
	set string "$string "
    }
 
    return $string 
}


proc WriteRealRange {word} {
    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    #writing and reading of range
    set string ""
    set location_hhyphen 0
    set location_comma 0
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i 0} {$i < $Length} {incr i} {
	
	set hyphen [string compare "-" [string index $word $i]]
	
	if {$hyphen == 0} {
	    set location_hhyphen $i
	    set location_comma [expr $location_comma+$hh]
	    #start value reading
	    for {set j $location_comma} {$j < $location_hhyphen} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 0
	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string   $start"
	    }
	} else {
	}
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set location_comma $i
	    set help [expr $location_hhyphen+1]
	    set help_comma [expr $help_comma+1]
	    
	    #end value reading
	    for {set j $help} {$j < $location_comma} {incr j} {
		set print [string index $word $j]
		set end "$end$print"
	    }
	    set hh 1
	    set string "$string $end"
	} else {
	}
		
	set start ""
	set end ""
    }
    set string "$X $string"
    
    return $string
}

proc WriteRealList {word} {

    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    set comma -1
    set ii 0
    set Error " "

    while {$comma != 0} {
	
	set tprint [string index $word $ii]
	set Error "$Error$tprint"
	
	set ii [expr $ii+1]
	set comma [string compare ":" [string index $word $ii]]
    }
    set ii [expr $ii+1]
    
    #writing and reading of range
    set string ""
    set next_location_comma 0
    set location_comma $ii
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i $ii} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set next_location_comma $i
	    set location_comma [expr $location_comma+$hh]
	    #start value
	    for {set j $location_comma} {$j < $next_location_comma} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 1
	    set location_comma $next_location_comma

	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string $start"
	    }
	} else {
	}
	
	set start ""
    }

    set string "$X  $string  $Error"
    
    return $string 
}


proc WriteTabFunct {word} {
    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    set string ""
    set location_hhyphen 0
    set location_comma 0
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i 0} {$i < $Length} {incr i} {
	
	set hyphen [string compare "-" [string index $word $i]]
	
	if {$hyphen == 0} {
	    set location_hhyphen $i
	    set location_comma [expr $location_comma+$hh]

	    for {set j $location_comma} {$j < $location_hhyphen} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 0

	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string   $start"
	    }

	} else {
	}
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set location_comma $i
	    set help [expr $location_hhyphen+1]
	    set help_comma [expr $help_comma+1]

	    for {set j $help} {$j < $location_comma} {incr j} {
		set print [string index $word $j]
		set end "$end$print"
	    }
	    set hh 1
	    set string "$string $end"
	} else {
	}
		
	set start ""
	set end ""
    }

    set string "$X $string"
    
    return $string
}

proc WriteNItimes {word} {

    set Length [string length $word]
    set X 0
    
    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set X [expr $X+1]
	}
    }
    
    set string ""
    set next_location_comma 0
    set location_comma 0
    set start ""
    set end ""
    set hh 0
    set LLength [expr $Length-1]
    set last_comma [string compare "," [string index $word $LLength]]
    set help_comma 0

    for {set i 0} {$i < $Length} {incr i} {
	
	set comma [string compare "," [string index $word $i]]
	
	if {$comma == 0} {
	    set next_location_comma $i
	    set location_comma [expr $location_comma+$hh]

	    for {set j $location_comma} {$j < $next_location_comma} {incr j} {
		set print [string index $word $j]
		set start "$start$print"
	    }
	    set hh 1
	    set location_comma $next_location_comma

	    if {$last_comma != 0 && $help_comma == $X} {
	    } else {
		set string "$string   $start"
	    }
	} else {
	}
	
	set start ""
    }

    set string "$X $string"
 
    return $string 
}

proc ProcessingConvert {} {

    set processc .gid.processc
    
    InitWindow $processc "Mesh converting" PROCESSC_MESH "" \
	"" 1
    
    frame $processc.top
    label $processc.top.title_text -text " Converting of element conectivity!!! "
   
    frame $processc.information  -relief ridge -bd 2 
    label $processc.information.problem1     -text " You have chosen converting of quadratic mesh into linear! "
    label $processc.information.problem2     -text " It will take long time"

    frame $processc.bottom
    button $processc.bottom.start -text "CONTINUE" \
           -height 1 -width 14 -command "destroy $processc"

    pack $processc.top.title_text -pady 10
    pack $processc.information.problem1 $processc.information.problem2 -side top -anchor w
    
    pack $processc.bottom.start -side left -anchor center
    pack $processc.top 
    pack $processc.information -expand yes -fill both
    pack $processc.bottom -side top -padx 6 -pady 10 -ipady 2
}
